---
jupytext:
  formats: md:myst
  text_representation:
    extension: .md
    format_name: myst
    format_version: 0.13
    jupytext_version: 1.14.1
kernelspec:
  display_name: Python 3 (ipykernel)
  language: python
  name: python3
---

# Cosinus Ähnlichkeit, Markdown

* J.Busse, www.jbusse.de, 2021-04-12, 2022-04-19, spaCy: 2022-06-09

Lizenz: public domain / [CC 0](https://creativecommons.org/publicdomain/zero/1.0/deed.de) 

Dieses Notebook: Erstes Eingangs-Beispiel für die LV dsci-txt SS 2021 HAW LA ... ein minimales Mini-Programm, das die Cosinus-Ähnlichkeit zwischen (Markdown-) Texten als seaborn-Clustermap visualisiert.

Beabsichtigte Verwendung:
* Studierende haben einen lauffähigen Ausgangspunkt,
* den sie hier und da spielerisch verändern können.

VEREINBARUNG: Wenn Code-Snippets im Netz recherchiert und nur unwesentlich verändert wurden, dann wird die Quelle in der ersten Zeile einer Zelle mit URL referenziert (siehe z.B. unten `glob`). Es liegt damit auch kein Plagiat mehr vor. Umgekehrt gilt: Eine Zelle, die keine Quellenangabe enhält, gilt als "aus dem Kopf" selbst geschrieben.

Dateien:
* {download}`regex_XX.zip`
* auspacken; das Verzeichnis `regex_XX/` sollte idealerweise als Geschwister-Verzeichnis des aktuellen Arbeitsverzeichnis `md/` angelegt werden.

```{code-cell} ipython3
import numpy as np
import pandas as pd
```

### Global Parameters

```{code-cell} ipython3
# path to files, incl. glob mask
#path_to_ipynb = "md/*.md"
path_to_ipynb = "../regex_XX/*.md"
```

```{code-cell} ipython3
# show intermediary results
# 0 none, 1 informative, 2 debug
verbosity = 1

def verbose(level,item):
    if level <= verbosity:
        display(item)
```

Read Files 
----

```{code-cell} ipython3
# https://stackoverflow.com/questions/3207219/how-do-i-list-all-files-of-a-directory
import glob
files = glob.glob(path_to_ipynb)
verbose(1,len(files))
verbose(2,files)
```

```{code-cell} ipython3
files
```

```{code-cell} ipython3
type(files[0])
```

```{code-cell} ipython3
corpus_dict = {}

for file in files:
    with open(file, 'r') as f:
        corpus_dict[file] = f.read()
corpus_dict
```

```{code-cell} ipython3
corpus = list(corpus_dict.values())
corpus
```

## spaCy

(2022-06-09)

Intro in spaCy: <https://spacy.io/usage/spacy-101>

```{code-cell} ipython3
import spacy

nlp = spacy.load("de_core_news_sm")
```

```{code-cell} ipython3
nlp(corpus[2])
```

```{code-cell} ipython3
def lemma_string(text, postags = None):
    """returns lemmata of a string, joined again to a string"""
    txt = nlp(text)
    txt_lemmata = [ t.lemma_ for t in txt if (not postags or t.pos_ in postags) ]
    return " ".join(txt_lemmata)
    
```

```{code-cell} ipython3
postags = ['NOUN', 'VERB', 'ADJ']
lemma_string(corpus[2], postags)
```

```{code-cell} ipython3
corpus_lemmatized = [ lemma_string(c, postags) for c in corpus ]
corpus_lemmatized[2]
```

```{code-cell} ipython3

```

Bag of Words: CountVectorizer
------

https://scikit-learn.org/stable/modules/generated/sklearn.feature_extraction.text.CountVectorizer.html#sklearn.feature_extraction.text.CountVectorizer

```{code-cell} ipython3
from sklearn.feature_extraction.text import CountVectorizer
count = CountVectorizer()

#bag_of_words = count.fit_transform(np.array(corpus))
bag_of_words = count.fit_transform(np.array(corpus_lemmatized))
verbose(2,type(bag_of_words))
```

```{code-cell} ipython3
bag_of_words
```

```{code-cell} ipython3
feature_names = count.get_feature_names()
len(feature_names)
```

```{code-cell} ipython3
pd.DataFrame(bag_of_words.toarray(), columns=feature_names).T
```

## TfIdf


https://chrisalbon.com/machine_learning/preprocessing_text/tf-idf/


https://scikit-learn.org/stable/modules/generated/sklearn.feature_extraction.text.TfidfVectorizer.html

Equivalent to CountVectorizer followed by TfidfTransformer: 

https://scikit-learn.org/stable/modules/generated/sklearn.feature_extraction.text.TfidfTransformer.html#sklearn.feature_extraction.text.TfidfTransformer



  * https://scikit-learn.org/stable/modules/generated/sklearn.feature_extraction.text.TfidfTransformer.html#sklearn.feature_extraction.text.TfidfTransformer
  * https://scikit-learn.org/stable/modules/feature_extraction.html#text-feature-extraction

```{code-cell} ipython3
from sklearn.feature_extraction.text import TfidfTransformer
transformer = TfidfTransformer(smooth_idf=False)
verbose(1,transformer)
```

```{code-cell} ipython3
tfidf = transformer.fit_transform(bag_of_words)
verbose(1,pd.DataFrame(tfidf.toarray()))
```

Cosine Similarity
-------

  * interessant, ggf. auch noch ausprobieren?:  https://stackoverflow.com/questions/17627219/whats-the-fastest-way-in-python-to-calculate-cosine-similarity-given-sparse-mat
  
Wir machen es hier eher low level, um unter die Motorhaube sehen zu können:
  * https://scikit-learn.org/stable/modules/metrics.html#cosine-similarity
  * didaktische Erklärung: http://blog.christianperone.com/2013/09/machine-learning-cosine-similarity-for-vector-space-models-part-iii/

```{code-cell} ipython3
from sklearn.metrics.pairwise import cosine_similarity
similarity = cosine_similarity(tfidf)
#verbose(2,similarity)
```

```{code-cell} ipython3
similarity_df = pd.DataFrame(similarity)
similarity_df.columns = files 
similarity_df.index = files
similarity_df
```

```{code-cell} ipython3
import seaborn as sns
mask = np.zeros_like(similarity)
mask[np.triu_indices_from(mask)] = True
#ax = sns.heatmap(similarity_df)
ax = sns.clustermap(similarity_df)
ax.savefig("clustermap.png")
```

https://docs.scipy.org/doc/scipy/reference/generated/scipy.cluster.hierarchy.linkage.html

```{code-cell} ipython3
corpus_dict['regex_XX/regex_8.md']
```

```{code-cell} ipython3
corpus_dict['regex_XX/regex_7.md']
```

```{code-cell} ipython3
corpus_dict['regex_XX/regex_wiki_1.md']
```

```{code-cell} ipython3

```
